@@ -4,7 +4,7 @@ use strict;
 use FileHandle;
 
 use vars qw( $VERSION );
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 =head1 NAME
 
@@ -44,11 +44,11 @@ sub new {
     my $handle = new FileHandle "<$file";
 
     my $self = {
-		'real_size'	=> $size,
-		'file'		=> $file,
-		'handle'	=> $handle,
-		'tools'		=> $tools,
-	       };
+	'real_size'	=> $size,
+	'file'		=> $file,
+	'handle'	=> $handle,
+	'tools'		=> $tools,
+    };
 
     bless $self, $class;
 
@@ -112,33 +112,33 @@ Returns the cuepoints marked within the wav file.
 
 Returns a reference to a hash containing;
 (for example, a file marked up for use in Audio::Mix)
-(position is byte offset)
+(position is sample position)
 
     {
-      1 => {
-	     label => 'sig',
-	     position => 764343,
-	     note => 'first'
-	   },
-      2 => {
-	     label => 'fade_in',
-	     position => 1661774,
-	     note => 'trig'
-	   },
-      3 => {
-	     label => 'sig',
-	     position => 18033735,
-	     note => 'last'
-	   },
-      4 => {
-	     label => 'fade_out',
-	     position => 17145150,
-	     note => 'trig'
-	   },
-      5 => {
-	     label => 'end',
-	     position => 18271676
-	   }
+	1 => {
+	    label	=> 'sig',
+	    position	=> 764343,
+	    note	=> 'first'
+	},
+	2 => {
+	    label	=> 'fade_in',
+	    position	=> 1661774,
+	    note	=> 'trig'
+	},
+	3 => {
+	    label	=> 'sig',
+	    position	=> 18033735,
+	    note	=> 'last'
+	},
+	4 => {
+	    label	=> 'fade_out',
+	    position	=> 17145150,
+	    note	=> 'trig'
+	},
+	5 => {
+	    label	=> 'end',
+	    position	=> 18271676
+	}
     }
 
 =cut
@@ -177,11 +177,26 @@ sub read_raw {
     return $self -> _read_raw( $len );
 }
 
+=head2 read_raw_samples
+
+Reads raw packed samples from the current audio data position in the file.
+
+    my $data = $self -> read_raw_samples( $samples );
+
+=cut
+
+sub read_raw_samples {
+    my $self = shift;
+    my $len = shift;
+    $len *= $self -> {'data'} -> {'block_align'};
+    return $self -> read_raw( $len );
+}
+
 sub _read_raw {
     my $self = shift;
     my $len = shift;
     my $data;
-    return undef unless $len;
+    return undef unless $len && $len > 0;
     $self -> {'pos'} += read( $self -> {'handle'}, $data, $len );
     return $data;
 }
@@ -221,14 +236,15 @@ sub _init_read_sub {
 	$sub = sub { return map $_ - $offset, unpack( 'C'.$channels, shift() ) };
     } else {
 	if ( $self -> {'tools'} -> is_big_endian() ) {
-	    $sub = sub { return unpack( 's'.$channels,				# 3. unpack native as signed short
-				    pack( 'S'.$channels,			# 2. pack native unsigned short
-					unpack( 'v'.$channels, shift() )	# 1. unpack little-endian unsigned short
-				    )
-				);
-		   };
+	    $sub = sub {
+		return unpack( 's' . $channels,			# 3. unpack native as signed short
+		    pack( 'S' . $channels,			# 2. pack native unsigned short
+			unpack( 'v' . $channels, shift() )	# 1. unpack little-endian unsigned short
+		    )
+		);
+	   };
 	} else {
-	    $sub = sub { return unpack( 's'.$channels, shift() ) };
+	    $sub = sub { return unpack( 's' . $channels, shift() ) };
 	}
     }
     $self -> {'read_sub'} = $sub;
@@ -247,6 +263,19 @@ sub position {
     return $self -> {'pos'} - $self -> {'data'} -> {'data_start'};
 }
 
+=head2 position_samples
+
+Returns the current audio data position (in samples).
+
+    my $samples = $read -> position_samples();
+
+=cut
+
+sub position_samples {
+    my $self = shift;
+    return ( $self -> {'pos'} - $self -> {'data'} -> {'data_start'} ) * $self -> {'data'} -> {'block_align'};
+}
+
 =head2 move_to
 
 Moves the current audio data position to byte offset.
@@ -258,7 +287,17 @@ Moves the current audio data position to byte offset.
 sub move_to {
     my $self = shift;
     my $pos = shift;
-    $pos = $self -> {'data'} -> {'data_start'} unless defined( $pos );
+    my $data_start = $self -> {'data'} -> {'data_start'};
+    if ( $pos ) {
+	$pos = 0 if $pos < 0;
+    } else {
+	$pos = 0;
+    }
+    $pos += $data_start;
+    if ( $pos > $self -> {'pos'} ) {
+        my $max_pos = $self -> reread_length() + $data_start;
+        $pos = $max_pos if $pos > $max_pos;
+    }
     if ( seek $self -> {'handle'}, $pos, 0 ) {
 	$self -> {'pos'} = $pos;
 	return 1;
@@ -339,6 +378,28 @@ sub details {
     return $self -> {'data'};
 }
 
+=head2 reread_length
+
+Rereads the length of the file in case it is being written to
+as we are reading it.
+
+    my $new_data_length = $read -> reread_length();
+
+=cut
+
+sub reread_length {
+    my $self = shift;
+    my $handle = $self -> {'handle'};
+    my $old_pos = $self -> {'pos'};
+    my $data = $self -> {'data'};
+    my $data_start = $data -> {'data_start'};
+    seek $handle, $data_start - 4, 0;
+    my $new_length = $self -> _read_long();
+    seek $handle, $old_pos, 0;
+    $data -> {'data_length'} = $new_length;
+    return $new_length;
+}
+
 #########
 
 sub _read_file {
@@ -358,81 +419,79 @@ sub _read_file {
 	return $self -> _error( "doesn't seem to be a wav file" );
     }
 
-	my $walkover;  # for fixing cooledit 96 data-chunk bug
+    my $walkover;  # for fixing cooledit 96 data-chunk bug
 
     while ( ! eof $handle && $self -> {'pos'} < $length ) {
-		my $head;
-		if ($walkover) {
-			# rectify cooledit 96 data-chunk bug
-			$head = $walkover . $self->_read_raw(3);
-			$walkover=undef;
-			print("debug: CoolEdit 96 data-chunk bug detected!\n") if $debug;
-		} else {
-			$head = $self -> _read_raw( 4 );
-		}
-		my $chunk_len = $self -> _read_long();
+	my $head;
+	if ( $walkover ) {
+	    # rectify cooledit 96 data-chunk bug
+	    $head = $walkover . $self -> _read_raw( 3 );
+	    $walkover = undef;
+	    print( "debug: CoolEdit 96 data-chunk bug detected!\n" ) if $debug;
+	} else {
+	    $head = $self -> _read_raw( 4 );
+	}
+	my $chunk_len = $self -> _read_long();
+	printf( "debug: head: '$head' at %6d (%6d bytes)\n", $self->{pos}, $chunk_len ) if $debug;
+	if ( $head eq 'fmt ' ) {
+	    my $format = $self -> _read_fmt( $chunk_len );
+	    my $comp = delete( $format -> {'format'} );
+	    unless ( $comp == 1 ) {
+		return $self -> _error( "seems to be compressed, I can't handle anything other than uncompressed PCM" );
+	    }
+	    %details = ( %details, %$format );
+	    next;
+	} elsif ( $head eq 'cue ' ) {
+	    $details{'cue'} = $self -> _read_cue( $chunk_len, \%details );
+	    next;
+	} elsif ( $head eq 'smpl' ) {
+	    $details{'sampler'} = $self -> _read_sampler( $chunk_len );
+	    next;
+	} elsif ( $head eq 'LIST' ) {
+	    my $list = $self -> _read_list( $chunk_len, \%details );
+	    next;
+	} elsif ( $head eq 'DISP' ) {
+	    $details{'display'} = $self -> _read_disp( $chunk_len );
+	    next;
+	} elsif ( $head eq 'data' ) {
+	    $details{'data_start'} = $self -> {'pos'};
+	    $details{'data_length'} = $chunk_len;
+	} else {
+	    $head =~ s/[^\w]+//g;
+	    $self -> _error( "ignored unknown block type: $head at $self->{pos} for $chunk_len", 'warn' );
+	    next if $chunk_len > 100;
+	}
 
-		printf("debug: head: '$head' at %6d (%6d bytes)\n",$self->{pos},$chunk_len) if $debug;
+	seek $handle, $chunk_len, 1;
+	$self -> {'pos'} += $chunk_len;
 
-		if ( $head eq 'fmt ' ) {
-		    my $format = $self -> _read_fmt( $chunk_len );
-		    my $comp = delete( $format -> {'format'} );
-		    unless ( $comp == 1 ) {
-			return $self -> _error( "seems to be compressed, I can't handle anything other than uncompressed PCM" );
-		    }
-		    %details = ( %details, %$format );
-		    next;
-		} elsif ( $head eq 'cue ' ) {
-		    $details{'cue'} = $self -> _read_cue( $chunk_len, \%details );
-		    next;
-		} elsif ( $head eq 'smpl' ) {
-		    $details{'sampler'} = $self -> _read_sampler( $chunk_len );
-		    next;
-		} elsif ( $head eq 'LIST' ) {
-		    my $list = $self -> _read_list( $chunk_len, \%details );
-		    next;
-		} elsif ( $head eq 'DISP' ) {
-		    $details{'display'} = $self -> _read_disp( $chunk_len );
-		    next;
-		} elsif ( $head eq 'data' ) {
-		    $details{'data_start'} = $self -> {'pos'};
-		    $details{'data_length'} = $chunk_len;
-		} else {
-		    $head =~ s/[^\w]+//g;
-		    $self -> _error( "ignored unknown block type: $head at $self->{pos} for $chunk_len", 'warn' );
-		    next if $chunk_len > 100;
-		}
+	# read padding
+	if ($chunk_len % 2) {
+	    my $pad = $self->_read_raw(1);
+	    if ( ($pad =~ /\w/) && $old_cooledit && ($head eq 'data') ) {
+		# Oh no, this file was written by cooledit 96...
+		# This is not a pad byte but the first letter of the next head.
+		$walkover = $pad;
+	    }
+	}
 
-		seek $handle, $chunk_len, 1;
-		$self -> {'pos'} += $chunk_len;
-
-		# read padding
-		if ($chunk_len % 2) {
-			my $pad = $self->_read_raw(1);
-			if ( ($pad =~ /\w/) && $old_cooledit && ($head eq 'data') ) {
-				# Oh no, this file was written by cooledit 96...
-				# This is not a pad byte but the first letter of the next head.
-				$walkover = $pad;
-			}
-		}
-
-		#unless ( $old_cooledit ) {
-		#    $chunk_len += 1 if $chunk_len % 2; # padding
-		#}
-		#seek $handle, $chunk_len, 1;
-		#$self -> {'pos'} += $chunk_len;
+	#unless ( $old_cooledit ) {
+	#    $chunk_len += 1 if $chunk_len % 2; # padding
+	#}
+	#seek $handle, $chunk_len, 1;
+	#$self -> {'pos'} += $chunk_len;
 
 
     }
 
     if ( exists $details{'data_start'} ) {
-		$details{'length'} = $details{'data_length'} / $details{'bytes_sec'};
-		$details{'data_finish'} = $details{'data_start'} + $details{'data_length'};
+	$details{'length'} = $details{'data_length'} / $details{'bytes_sec'};
+	$details{'data_finish'} = $details{'data_start'} + $details{'data_length'};
     } else {
-		$details{'data_start'} = 0;
-		$details{'data_length'} = 0;
-		$details{'length'} = 0;
-		$details{'data_finish'} = 0;
+	$details{'data_start'} = 0;
+	$details{'data_length'} = 0;
+	$details{'length'} = 0;
+	$details{'data_finish'} = 0;
     }
     return \%details;
 }
@@ -531,25 +590,25 @@ sub _read_sampler {
     my $record = $self -> _decode_block( $sampler_fields{'fields'} );
 
     for my $id ( 1 .. $record -> {'sample_loops'} ) {
-		push @{ $record -> {'loop'} }, $self -> _decode_block( $sampler_fields{'loop'} );
+	push @{ $record -> {'loop'} }, $self -> _decode_block( $sampler_fields{'loop'} );
     }
 
-	$record -> {'sample_specific_data'} = _read_raw($record -> {'sample_data'});
+    $record -> {'sample_specific_data'} = _read_raw( $record -> {'sample_data'} );
 
-	my $read_bytes =
-		  9*4								# sampler info
-		+ 6*4*$record->{'sample_loops'}		# loops
-		+ $record->{'sample_data'};			# specific data
+    my $read_bytes =
+	9 * 4					# sampler info
+	+ 6 * 4 * $record -> {'sample_loops'}	# loops
+	+ $record -> {'sample_data'};		# specific data
 
 
-	# read any junk
-	if ($read_bytes  < $length) {
-		my $junk = $self->_read_raw( $length-$read_bytes );
-	}
+    # read any junk
+    if ($read_bytes < $length ) {
+	my $junk = $self->_read_raw( $length - $read_bytes );
+    }
 
-	if ($length % 2) {
-		my $pad = $self->_read_raw(1);
-	}
+    if ( $length % 2 ) {
+	my $pad = $self -> _read_raw( 1 );
+    }
 
     # temporary nasty hack to gooble the last bogus 12 bytes
     #my $extra = $self -> _decode_block( $sampler_fields{'extra'} );
@@ -614,7 +673,7 @@ sub _error {
 
 =head1 AUTHORS
 
-    Nick Peskett <cpan@peskett.com>.
+    Nick Peskett (see http://www.peskett.co.uk/ for contact details).
     Kurt George Gjerde <kurt.gjerde@media.uib.no>. (from 0.02)
 
 =cut
