@@ -1,8 +1,9 @@
 package Audio::Wav::Write::Header;
 
 use strict;
-use vars qw($VERSION);
-$VERSION = '0.03';
+
+use vars qw( $VERSION );
+$VERSION = '0.04';
 
 sub new {
     my $class = shift;
@@ -12,14 +13,14 @@ sub new {
     my $handle = shift;
     my $parent = shift;
     my $self = {
-		'file'		=> $file,
-		'data'		=> undef,
-		'details'	=> $details,
-		'tools'		=> $tools,
-		'handle'	=> $handle,
-		'whole_offset'	=> 4,
-		'parent'	=> $parent,
-	       };
+	'file'		=> $file,
+	'data'		=> undef,
+	'details'	=> $details,
+	'tools'		=> $tools,
+	'handle'	=> $handle,
+	'whole_offset'	=> 4,
+	'parent'	=> $parent,
+    };
     bless $self, $class;
     return $self;
 }
@@ -173,7 +174,7 @@ sub _write_list_info {
     my %allowed = $self -> {'tools'} -> get_rev_info_fields();
     my $list='INFO';
     foreach my $key ( keys %$info ) {
-		next unless $allowed{$key};  # don't write unknown info-chunks
+	next unless $allowed{$key};  # don't write unknown info-chunks
         $list .= $self -> _make_chunk( $allowed{$key}, $info -> {$key} . "\0" );
     }
     return $self -> _write_block( 'LIST', $list );
@@ -184,20 +185,20 @@ sub _write_cues {
     return 0 unless $self -> {'cues'};
     my $cues = $self -> {'cues'};
     my @fields = qw( id position chunk cstart bstart offset );
-    my %plain = map { $_, 1 } qw( chunk );
+    my %plain = ( 'chunk' => 1 );
     my %defaults;
     my $output = pack( 'V', scalar( @$cues ) );
     foreach my $id ( 0 .. $#$cues ) {
 	my $cue = $cues -> [$id];
 	my $pos = $cue -> {'pos'};
-	my %record =	(
-			'id'		=> $id + 1,
-			'position'	=> $pos,
-			'chunk'		=> 'data',
-			'cstart'	=> 0,
-			'bstart'	=> 0,
-			'offset'	=> $pos,
-			);
+	my %record = (
+	    'id'	=> $id + 1,
+	    'position'	=> $pos,
+	    'chunk'	=> 'data',
+	    'cstart'	=> 0,
+	    'bstart'	=> 0,
+	    'offset'	=> $pos,
+	);
 	foreach my $field ( @fields ) {
 	    my $data = $record{$field};
 	    $data = pack( 'V', $data ) unless exists( $plain{$field} );
@@ -279,4 +280,3 @@ sub _error {
 }
 
 1;
-
